/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2017  SEGGER Microcontroller GmbH & Co. KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

** emWin V5.46 - Graphical user interface for embedded applications **
All  Intellectual Property rights  in the Software belongs to  SEGGER.
emWin is protected by  international copyright laws.  Knowledge of the
source code may not be used to write a similar product.  This file may
only be used in accordance with the following terms:

The software  has been licensed to  Cypress Semiconductor Corporation,
whose registered  office is situated  at 198 Champion Ct. San Jose, CA 
95134 USA  solely for the  purposes of creating  libraries for Cypress
PSoC3 and  PSoC5 processor-based devices,  sublicensed and distributed
under  the  terms  and  conditions  of  the  Cypress  End User License
Agreement.
Full source code is available at: www.segger.com

We appreciate your understanding and fairness.
----------------------------------------------------------------------
Licensing information
Licensor:                 SEGGER Microcontroller Systems LLC
Licensed to:              Cypress Semiconductor Corp, 198 Champion Ct., San Jose, CA 95134, USA
Licensed SEGGER software: emWin
License number:           GUI-00319
License model:            Services and License Agreement, signed June 10th, 2009
Licensed platform:        Any Cypress platform (Initial targets are: PSoC3, PSoC5)
----------------------------------------------------------------------
Support and Update Agreement (SUA)
SUA period:               2009-06-12 - 2022-07-27
Contact to extend SUA:    sales@segger.com
----------------------------------------------------------------------
File        : LCDConf.c
Purpose     : Display controller configuration (single layer)
---------------------------END-OF-HEADER------------------------------
*/

#include <project.h>
#include "GUI.h"
#include "LCDConf.h"
#include "GUIConf.h"


/*********************************************************************
*       Layer configuration
*********************************************************************/

/* Physical display size */
#define XSIZE_PHYS          320
#define YSIZE_PHYS          240

/* Virtual display size */
#define VXSIZE_PHYS          320
#define VYSIZE_PHYS          240

/* Color conversion */
#define COLOR_CONVERSION    GUICC_M565

/* Display driver */
#define DISPLAY_DRIVER GUIDRV_TEMPLATE

#define NUM_BUFFERS 3

#define BITSPERPIXEL 2

#define FRAME_BUFFER_SIZE (XSIZE_PHYS * YSIZE_PHYS * BITSPERPIXEL)

int pendingBufferindex = -1;

extern void _CopyBuffer(GUI_DEVICE * pDevice, int Index, int LayerIndex, int IndexSrc, int IndexDst);

#define GUI_TOUCH_AD_TOP 2425
#define GUI_TOUCH_AD_BOTTOM 229
#define GUI_TOUCH_AD_LEFT 2190
#define GUI_TOUCH_AD_RIGHT 173

/*********************************************************************
*       Configuration checking
*********************************************************************/

#ifndef   VXSIZE_PHYS
  #define VXSIZE_PHYS XSIZE_PHYS
#endif
#ifndef   VYSIZE_PHYS
  #define VYSIZE_PHYS YSIZE_PHYS
#endif
#ifndef   XSIZE_PHYS
  #error Physical X size of display is not defined!
#endif
#ifndef   YSIZE_PHYS
  #error Physical Y size of display is not defined!
#endif
#ifndef   VRAM_ADDR
  #define VRAM_ADDR 0x20000000
#endif
#ifndef   COLOR_CONVERSION
  #error Color conversion not defined!
#endif
#ifndef   DISPLAY_DRIVER
  #error No display driver defined!
#endif
#if LCD_SWAP_XY
  #define LCD_XSIZE YSIZE_PHYS
  #define LCD_YSIZE XSIZE_PHYS
  #define LCD_VXSIZE VYSIZE_PHYS
  #define LCD_VYSIZE VXSIZE_PHYS
#else
  #define LCD_XSIZE XSIZE_PHYS
  #define LCD_YSIZE YSIZE_PHYS
  #define LCD_VXSIZE VXSIZE_PHYS
  #define LCD_VYSIZE VYSIZE_PHYS
#endif

GUI_DEVICE *device;


void LXD_Init(void)
{
    LCD_Reset_Write(1);
    
    // Vcom setting.
    SPIM_1_WriteTxData(0x028);		
    SPIM_1_WriteTxData(0x100);
    SPIM_1_WriteTxData(0x106);

    // Oscillation Start - Turn SSD2119 Internal Oscillator on.
    SPIM_1_WriteTxData(0x000);		
    SPIM_1_WriteTxData(0x100);
    SPIM_1_WriteTxData(0x101);

    // VCOM OTP - Leave Sleep Mode.
    SPIM_1_WriteTxData(0x010);		
    SPIM_1_WriteTxData(0x100);
    SPIM_1_WriteTxData(0x100);
    	
    CyDelay(1000);

    // LCD Drive AC - Normal.
    SPIM_1_WriteTxData(0x002);		
    SPIM_1_WriteTxData(0x100);
    SPIM_1_WriteTxData(0x100);

    // Power Control (1) - Set-up cycle and set-up factor.
    SPIM_1_WriteTxData(0x003);		
    SPIM_1_WriteTxData(0x162);
    SPIM_1_WriteTxData(0x134);

    // Driver Output Control 
    SPIM_1_WriteTxData(0x001);		
    SPIM_1_WriteTxData(0x172);
    SPIM_1_WriteTxData(0x1EF);

    // Entry Mode 
    SPIM_1_WriteTxData(0x011);		
    SPIM_1_WriteTxData(0x16F);// It was 47 and we changed it to 67
    SPIM_1_WriteTxData(0x130);

    // Power Control (2) - Adjust VCIX2 output voltage to 5.5V.
    SPIM_1_WriteTxData(0x00c);		
    SPIM_1_WriteTxData(0x100);
    SPIM_1_WriteTxData(0x103);	

    // Power Control (3) - Set amplitude magnification of VLCD63 to Vref X 2.335.
    SPIM_1_WriteTxData(0x00d);		
    SPIM_1_WriteTxData(0x100);
    SPIM_1_WriteTxData(0x10A);

    // Power Control (4) - Set the alternating amplitudes of Vcom to VLCD63 x 0.90.
    SPIM_1_WriteTxData(0x00E);		
    SPIM_1_WriteTxData(0x12A);
    SPIM_1_WriteTxData(0x100);

    // Power Control (5) - nOTP set to 0.
    SPIM_1_WriteTxData(0x01E);		
    SPIM_1_WriteTxData(0x100);
    SPIM_1_WriteTxData(0x12B);

    // Gamma Control (1) 
    SPIM_1_WriteTxData(0x030);		
    SPIM_1_WriteTxData(0x100);
    SPIM_1_WriteTxData(0x100);

    // Gamma Control (2)
    SPIM_1_WriteTxData(0x031);	
    SPIM_1_WriteTxData(0x105);
    SPIM_1_WriteTxData(0x104); 

    // Gamma Control (3)
    SPIM_1_WriteTxData(0x032);	
    SPIM_1_WriteTxData(0x101);
    SPIM_1_WriteTxData(0x104);

    // Gamma Control (4)
    SPIM_1_WriteTxData(0x033);	
    SPIM_1_WriteTxData(0x103);
    SPIM_1_WriteTxData(0x105);

    // Gamma Control (5)
    SPIM_1_WriteTxData(0x034);	
    SPIM_1_WriteTxData(0x105);
    SPIM_1_WriteTxData(0x105);

    // Gamma Control (6)
    SPIM_1_WriteTxData(0x035);	
    SPIM_1_WriteTxData(0x104);
    SPIM_1_WriteTxData(0x101);

    // Gamma Control (7)
    SPIM_1_WriteTxData(0x036);	
    SPIM_1_WriteTxData(0x103);
    SPIM_1_WriteTxData(0x103);

    // Gamma Control (8)
    SPIM_1_WriteTxData(0x037);	
    SPIM_1_WriteTxData(0x103);
    SPIM_1_WriteTxData(0x100);

    // Gamma Control (9)
    SPIM_1_WriteTxData(0x03A);	
    SPIM_1_WriteTxData(0x112);
    SPIM_1_WriteTxData(0x100);

    // Gamma Control (10)
    SPIM_1_WriteTxData(0x03B);	
    SPIM_1_WriteTxData(0x108);
    SPIM_1_WriteTxData(0x100);

    // Display Control - Operation, Grayscale level output, Selected gate line: VGH Non-selected gate line: VGOFFL.
    SPIM_1_WriteTxData(0x007);			
    SPIM_1_WriteTxData(0x100);
    SPIM_1_WriteTxData(0x133);
/********************************************************/
    // Vertical RAM address position.
    SPIM_1_WriteTxData(0x044);		
    SPIM_1_WriteTxData(0x1EF); // changed EF
    SPIM_1_WriteTxData(0x100); // changed 00	
/********************************************************/
    // Horizontal RAM address start position.
    SPIM_1_WriteTxData(0x045);		
    SPIM_1_WriteTxData(0x100); // changes 00 
    SPIM_1_WriteTxData(0x100); // changes 00 

    // Horizontal RAM address end position.
    SPIM_1_WriteTxData(0x046);	
    SPIM_1_WriteTxData(0x101); // changes 01 
    SPIM_1_WriteTxData(0x13F); // changes 3F 	

    // GDRAM X address counter. 
    SPIM_1_WriteTxData(0x04E);		
    SPIM_1_WriteTxData(0x100);
    SPIM_1_WriteTxData(0x100);
    	
    // GDRAM Y address counter.
    SPIM_1_WriteTxData(0x04F);	
    SPIM_1_WriteTxData(0x100);
    SPIM_1_WriteTxData(0x100);	

    // Frame Frequency - Set the frame frequency to 65Hz.
    SPIM_1_WriteTxData(0x025);		
    SPIM_1_WriteTxData(0x150);
    SPIM_1_WriteTxData(0x100);

//     Frame Cycle Control - Set non-overlap of the gate output to 1 clock cycle, 
//     set delay amount from the gate output signal falling edge of the source output to 0 clock cycle,
//     set the equalizing period to 3 clock cycle,
//     set RTN to count,
//     set count to 0x0B.
    SPIM_1_WriteTxData(0x00B);		
    SPIM_1_WriteTxData(0x142);
    SPIM_1_WriteTxData(0x11B);
    
//    SPIM_1_WriteTxData(0x016);		
//    SPIM_1_WriteTxData(0x100);
//    SPIM_1_WriteTxData(0x11E);
//    
//    SPIM_1_WriteTxData(0x017);		
//    SPIM_1_WriteTxData(0x100);
//    SPIM_1_WriteTxData(0x109);
    
//    GUI_Delay(2);
//    LCD_Reset_Write(1);
}

/*********************************************************************
*
*       Static code
*
*********************************************************************/

/*********************************************************************
*
*       LCD_X_InitController
*
* Purpose:
*   Initializes the display controller
*/
static void _InitController(void){
    /* Set up the display controller and put it into operation. If the 
    *  display controller is not initialized by any external routine 
    *  this needs to be adapted by the customer.
    */
    PWM_1_Start();
    
    CYGRAPHICS_START();
    SPIM_1_Start();
    ResistiveTouch_1_Start(); 
    LXD_Init();
    Tick_5ms_StartEx(Tick_5ms_Interrupt);
    Timer_5ms_Start();
    
    
    
    
    
    CYGRAPHICS_WRITE_FRAME(0);
    CYGRAPHICS_WRITE_INCR(VXSIZE_PHYS);
}

/*********************************************************************
*
*       _SetOrg
*
* Purpose:
*   Set the origin of the display by modifying the frame buffer base 
*   address register
*/
static void _SetOrg(int yPos, int xPos) {
  int x, y;
  #if   (!LCD_MIRROR_X && !LCD_MIRROR_Y && !LCD_SWAP_XY)
    #define LOG2PHYS_X(x, y) x
    #define LOG2PHYS_Y(x, y) y
  #elif (!LCD_MIRROR_X && !LCD_MIRROR_Y &&  LCD_SWAP_XY)
    #define LOG2PHYS_X(x, y) y
    #define LOG2PHYS_Y(x, y) x
  #elif (!LCD_MIRROR_X &&  LCD_MIRROR_Y && !LCD_SWAP_XY)
    #define LOG2PHYS_X(x, y) x
    #define LOG2PHYS_Y(x, y) LCD_VYSIZE - LCD_YSIZE - (y)
  #elif (!LCD_MIRROR_X &&  LCD_MIRROR_Y &&  LCD_SWAP_XY)
    #define LOG2PHYS_X(x, y) y
    #define LOG2PHYS_Y(x, y) LCD_VXSIZE - LCD_XSIZE - (x)
  #elif ( LCD_MIRROR_X && !LCD_MIRROR_Y && !LCD_SWAP_XY)
    #define LOG2PHYS_X(x, y) LCD_VXSIZE - LCD_XSIZE - (x)
    #define LOG2PHYS_Y(x, y) y
  #elif ( LCD_MIRROR_X && !LCD_MIRROR_Y &&  LCD_SWAP_XY)
    #define LOG2PHYS_X(x, y) LCD_VYSIZE - LCD_YSIZE - (y)
    #define LOG2PHYS_Y(x, y) x
  #elif ( LCD_MIRROR_X &&  LCD_MIRROR_Y && !LCD_SWAP_XY)
    #define LOG2PHYS_X(x, y) LCD_VXSIZE - LCD_XSIZE - (x)
    #define LOG2PHYS_Y(x, y) LCD_VYSIZE - LCD_YSIZE - (y)
  #elif ( LCD_MIRROR_X &&  LCD_MIRROR_Y &&  LCD_SWAP_XY)
    #define LOG2PHYS_X(x, y) LCD_VYSIZE - LCD_YSIZE - (y)
    #define LOG2PHYS_Y(x, y) LCD_VXSIZE - LCD_XSIZE - (x)
  #endif

  x = LOG2PHYS_X(xPos, yPos);
  y = LOG2PHYS_Y(xPos, yPos);
  
  while(CYGRAPHICS_IS_VBLANKING){
    /* Wait until an active period before changing the frame buffer
    *  address to be sure that the complete address will be updated
    *  before it is used.
    */    
  }
  CYGRAPHICS_WRITE_FRAME(x + y *VXSIZE_PHYS);
}

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/

/*********************************************************************
*
*       LCD_X_Config
*
* Purpose:
*   Called during the initialization process in order to set up the
*   display driver configuration.
*
*/
void LCD_X_Config(void) {
   
  //
  // Initialize MultiBuffering
  //
  GUI_MULTIBUF_Config(NUM_BUFFERS);
    
  /* Set display driver and color conversion */
  device = GUI_DEVICE_CreateAndLink(DISPLAY_DRIVER, COLOR_CONVERSION, 0, 0);
  
  /* Display driver configuration */
  
  LCD_SetSizeEx (0, LCD_XSIZE,   LCD_YSIZE);
  LCD_SetVSizeEx(0, LCD_VXSIZE,  LCD_VYSIZE);
  LCD_SetVRAMAddrEx(0, (void*)VRAM_ADDR);

  LCD_SetDevFunc(0, LCD_DEVFUNC_COPYBUFFER, (void (*)())_CopyBuffer);


 int TouchOrientation = (GUI_MIRROR_X * LCD_GetMirrorX()) |
                        (GUI_MIRROR_Y * LCD_GetMirrorY()) |
                        (GUI_SWAP_XY ) ;
//GUI_MIRROR_X | GUI_MIRROR_Y;
    
                    
  GUI_TOUCH_SetOrientation(TouchOrientation);


  GUI_TOUCH_Calibrate(GUI_COORD_Y, 0, 240, GUI_TOUCH_AD_TOP , GUI_TOUCH_AD_BOTTOM);
  GUI_TOUCH_Calibrate(GUI_COORD_X, 0, 320, GUI_TOUCH_AD_LEFT, GUI_TOUCH_AD_RIGHT);

  
}
/*********************************************************************
*
*       LCD_X_DisplayDriver
*
* Purpose:
*   This function is called by the display driver for several purposes.
*   To support the according task, the routine needs to be adapted to
*   the display controller. Note that the commands marked
*   "optional" are not cogently required and should only be adapted if 
*   the display controller supports these features.
*
* Parameter:
*   LayerIndex - Zero based layer index
*   Cmd        - Command to be executed
*   pData      - Pointer to a data structure.
* 
* Return Value:
*   < -1 - Error
*     -1 - The command is not handled.
*      0 - OK.
*/
int LCD_X_DisplayDriver(unsigned LayerIndex, unsigned Cmd, void * pData) {
  int r  = 0;
//  GUI_USE_PARA(LayerIndex);
  LCD_X_SHOWBUFFER_INFO *bufferInfo;
  unsigned long BufferSize;
  unsigned long Addr;

  switch (Cmd) {
  //
  // Required
  //
  case LCD_X_INITCONTROLLER: {
    //
    // Called during the initialization process in order to set up the
    // display controller and put it into operation. If the display
    // controller is not initialized by any external routine, this needs
    // to be adapted by the customer...
    //
    _InitController();
    r = 0;
    break;
    }
    
    case LCD_X_SETVRAMADDR: {
    //
    // Required for setting the address of the video RAM for drivers
    // with memory mapped video RAM which is passed in the 'pVRAM' element of p
    //
    LCD_X_SETVRAMADDR_INFO * p;
    p = (LCD_X_SETVRAMADDR_INFO *)pData;
    CYGRAPHICS_WRITE_FRAME((uint32)p->pVRAM);
    return 0;
  }
    
    case LCD_X_SETORG: 
    {
        // Required for setting the display origin which is passed 
        // in the 'xPos' and 'yPos' element of p
        
        LCD_X_SETORG_INFO * p;
        p = (LCD_X_SETORG_INFO *)pData;
        _SetOrg(p->xPos, p->yPos);
        return 0;
        
    }
    case LCD_X_SHOWBUFFER: {
        
        bufferInfo = (LCD_X_SHOWBUFFER_INFO *)pData;
        //
        // Calculate address of the given buffer
        //
        BufferSize = (FRAME_BUFFER_SIZE) / 8;
        Addr = 0x000000u + BufferSize * bufferInfo->Index;
        //
        // Make the given buffer visible
        //
        CYGRAPHICS_WRITE_FRAME(Addr);
        //
        // Send a confirmation that the buffer is visible now
        //
        pendingBufferindex = bufferInfo->Index;
        GUI_MULTIBUF_Confirm(pendingBufferindex);
        break;
    }
  
  default:
    r = -1;
  }
  return r;
}

/*************************** End of file ****************************/
